/**********************************************************************
 * Project: AderPlotter
 * Author: Andrew Deren <andrew@adersoftware.com>
 * http://www.adersoftware.com
 * Date: Feb. 2004
 * 
 * This program can be used for educational purposes only. 
 * For any other uses, please contact the author.
 **********************************************************************/

using System;
using System.Collections;
using System.Windows.Forms;
using System.Drawing;

namespace AderPlotter.Controls
{
	public interface IPlotter
	{
		void Init();

		/// <summary>
		/// given x returns y value for x
		/// </summary>
		float GetValue(float x);

		Color PlotColor
		{
			get;
			set;
		}
	}

	/// <summary>
	/// Summary description for Plotter.
	/// </summary>
	public class Plotter : Control
	{
		float startX;
		float endX;

		// bottom of screen is startY
		float startY;

		// top of screen is endY
		float endY;

		ArrayList plotters; // list of IPlotter

		IPlotter selected;

		public Plotter()
		{
			//
			// TODO: Add constructor logic here
			//
			this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			this.SetStyle(ControlStyles.DoubleBuffer, true);
			this.SetStyle(ControlStyles.ResizeRedraw, true);
			this.SetStyle(ControlStyles.UserPaint, true);
			this.UpdateStyles();

			this.startX = -10f;
			this.endX = 10f;

			this.startY = -1f;
			this.endY = 2f;

			this.plotters = new ArrayList();

		}

		public IPlotter SelectedPlotter
		{
			get { return this.selected; }
			set { this.selected = value; Invalidate(); }
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			Graphics g = e.Graphics;

			g.Clear(this.BackColor);

			float dx = PixelWidth;
			float dy = PixelHeight;


			foreach (IPlotter plotter in plotters)
			{
				float x = startX;
				float lastY = 0;
				int lastDrawY = 0;

				Pen pen = new Pen(plotter.PlotColor);

				if (plotter == selected)
					pen.Width = 2.0f;

				for (int drawX=0; drawX<Width; drawX++)
				{
					float y = 0;
					try
					{
						y = checked(plotter.GetValue(x));
					}
					catch (System.OverflowException)
					{
						y = 0;
					}

					int drawY = Height - (int)((y - startY) / dy);

					if (drawY < 0)
						drawY = -1;
					else if (drawY > Height)
						drawY = Height+1;

					if (drawX > 0)
					{
						float sy = lastDrawY;
						float ey = drawY;

						if (Single.IsNaN(lastY))
							sy = ey;
						if (Single.IsNaN(y))
							ey = sy;

						g.DrawLine(pen, drawX-1, sy, drawX, ey);
					}

					lastDrawY = drawY;
					lastY = y;
					x += dx;

				}

				pen.Dispose();

			}

			DrawAxis(g);
		}

		private string FormatLabel(float value)
		{
			value = Math.Abs(value);

			if (value > 10)
				return value.ToString();
			else if (value > 0.1)
				return value.ToString("F2");
			else if (value > 0.01)
				return value.ToString("F3");
			else
				return value.ToString("F4");

		}

		private void DrawAxis(Graphics g)
		{
			float dx = PixelWidth;
			float dy = PixelHeight;

			StringFormat format = new StringFormat();

			int zeroY = Height-(int)((0-startY) / dy);
			g.DrawLine(Pens.Black, 0, zeroY, Width, zeroY);

			int zeroX = (int)((0-startX) / dx);
			g.DrawLine(Pens.Black, zeroX, 0, zeroX, Height);

			format.Alignment = StringAlignment.Center;
			format.LineAlignment = StringAlignment.Near;

			float rangeX = endX - startX;
			float stepX = 1.0f;

			if (rangeX >= 20)
			{
				stepX = (float)Math.Ceiling(rangeX / 20);
			}
			else
			{
				stepX = rangeX / 20;
				stepX = (float)(Math.Round(stepX * 100f) / 100f);
			}

			for (float i=stepX; ; i+=stepX)
			{
				int drawX = (int)((i-startX) / dx);
				if (drawX > Width)
					break;

				g.DrawLine(Pens.Black, drawX, zeroY-3, drawX, zeroY+3);

				g.DrawString(FormatLabel(i), Font, Brushes.Black, drawX, zeroY+4, format);
			}
			for (float i = 0-stepX; ; i-=stepX)
			{
				int drawX = (int)((i-startX) / dx);
				if (drawX < 0)
					break;

				g.DrawLine(Pens.Black, drawX, zeroY-3, drawX, zeroY+3);
				g.DrawString(FormatLabel(i), Font, Brushes.Black, drawX, zeroY+4, format);
			}

			format.Alignment = StringAlignment.Far;
			format.LineAlignment = StringAlignment.Center;

			float rangeY = endY - startY;
			float stepY = 1.0f;

			if (rangeY >= 20)
			{
				stepY = (float)Math.Ceiling(rangeY / 20);
			}
			else
			{
				stepY = rangeY / 20;
				stepY = (float)(Math.Round(stepY * 100f) / 100f);
			}

			
			for (float i=stepY; ; i+=stepY)
			{
				int drawY = Height - (int)((i-startY) / dy);
				if (drawY < 0)
					break;

				g.DrawLine(Pens.Black, zeroX-3, drawY, zeroX+3, drawY);
				g.DrawString(FormatLabel(i), Font, Brushes.Black, zeroX-4, drawY, format);
			}

			for (float i=0-stepY; ; i-=stepY)
			{
				int drawY = Height - (int)((i-startY) / dy);
				if (drawY > Height)
					break;

				g.DrawLine(Pens.Black, zeroX-3, drawY, zeroX+3, drawY);
				g.DrawString(FormatLabel(i), Font, Brushes.Black, zeroX-4, drawY, format);
			}

		}

		private float PixelWidth
		{
			get
			{
				return (endX - startX) / (float)Width;
			}
		}

		private float PixelHeight
		{
			get
			{
				return (endY - startY) / (float)Height;
			}
		}

		public float StartX
		{
			get { return this.startX; }
			set { this.startX = value; Invalidate();}
		}

		public float EndX
		{
			get { return this.endX; }
			set { this.endX = value; Invalidate();}
		}

		public float StartY
		{
			get { return this.startY; }
			set { this.startY = value; Invalidate();}
		}

		public float EndY
		{
			get { return this.endY; }
			set { this.endY = value; Invalidate();}
		}

		public void AddPlotter(IPlotter plotter)
		{
			this.plotters.Add(plotter);
			Invalidate();
		}

		public int PlotterCount
		{
			get { return this.plotters.Count; }
		}

		public IPlotter PlotterAt(int pos)
		{
			return plotters[pos] as IPlotter;
		}
		
		public void RemovePlotter(int pos)
		{
			plotters.RemoveAt(pos);
			Invalidate();
		}
	}
}
